<?php
namespace Wordpress\Plugins\CarDealerPress\Inventory\Api;

class cdp_multi_request {

	private $curl_url;
	private $company_id;
	private $dealer_id;
	private $per_page_limit = 50;
	private $DBinventory_version = '5.3.200101';
	private $DBcompany_version = '5.4.200704';
	private $retry_limit = 5;
	private $retry = 0;
	private $blog = 1;

	function __construct(){
		global $wpdb;
		$this->wpdb = $wpdb;
		$this->set_option_variables();
		$this->create_company_table();
		$this->create_inventory_table();
		$this->create_data_table();

		if( $this->company_id ){
			$this->disclaimer_check();
		}
	}

	function set_option_variables(){
		$this->blog = get_current_blog_id();
		$this->JobStartTime = current_time( 'timestamp', 0 );
		$this->settings = get_option( 'cardealerpress_settings' );
		$this->database_inventory = $this->settings['database']['inventory'];
		$this->database_company = $this->settings['database']['company'];
		$this->company_id = ( isset($this->settings['vehicle_management_system']['company_information']['id']) || !empty($this->settings['vehicle_management_system']['company_information']['id'])? $this->settings['vehicle_management_system']['company_information']['id']:0 );
		$this->sortData = ( isset($this->settings['vehicle_management_system']['sort']['data'])? $this->settings['vehicle_management_system']['sort']['data']:  array() );
		$this->runPurge = FALSE;
		$this->api_call = urldecode($this->settings['vehicle_management_system']['host']);
	}

	function company_check(){
		if( false === (get_transient('cdp_company_check')) ){

			$query = 'select * from cdp_companies where blog_id = "'.$this->blog.'" and company_id = "'.$this->company_id.'" AND dealer_id = "'.$this->company_id.'" ';
			$results = $this->wpdb->get_results($query, ARRAY_A);
			if( empty($results) ){
				error_log('CDP Import: Adding new dealer '.$this->company_id);
				set_transient('cdp_company_check', 'Waiting 12 hours before next check.', 3600 * 12 );
				$this->init_dealer_call();
				$this->add_cdp_data_dealer();
				$this->add_manual_dealers();
				$this->remove_invalid_vehicles();
				$this->company_duplicate_check();
			} else {
				error_log('CDP Company Checked. Next Check in 12 Hours');
				$api_params = '/api/companies/'.$this->company_id;
				$response = $this->http_get_file($api_params);
				$response_code = wp_remote_retrieve_response_code($response);
				if( $response_code == 200 ){
					$this->dealer = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
					if( isset($this->dealer->updated) ){
						set_transient('cdp_company_check', 'Waiting 12 hours before next check.', 3600 * 12 );
						if( $results[0]['last_updated'] < strtotime($this->dealer->updated) ){
							error_log('CDP Import: Updating dealer '.$this->company_id);
							$this->wpdb->query('DELETE from cdp_companies where blog_id="'.$this->blog.'"');
							$this->init_dealer_call();
							$this->add_cdp_data_dealer();
							$this->add_manual_dealers();
							$this->remove_invalid_vehicles();
							$this->company_duplicate_check();
						}
					}
				}
			}
		}
	}

	function company_duplicate_check(){

		$query = 'DELETE FROM cdp_companies WHERE id in ( SELECT max_id from ( SELECT max(id) as max_id FROM cdp_companies WHERE dealer_id IN ( SELECT dealer_id FROM (SELECT dealer_id, count(*) AS duplicates FROM cdp_companies WHERE company_id = '.$this->company_id.' AND blog_id = '.$this->blog.' GROUP BY dealer_id ) AS z where z.duplicates > 1 ) GROUP BY id ) AS a )';
		$sql_results = $this->wpdb->query($query);
		if( $sql_results > 0 ){
			error_log('Company Duplicates Removed');
		}

	}

	function disclaimer_check(){

		$query = 'Select * from cdp_data where blog_id = "'.$this->blog.'" and data_key = "disclaimer" ';
		$results = $this->wpdb->get_results($query, ARRAY_A);
		if( empty($results) ){
			$api_params = '/'.$this->company_id.'/inventory/vehicles.json?per_page=1';
			$response = $this->http_get_file($api_params);
			$response_code = wp_remote_retrieve_response_code($response);
			if( $response_code == 200 ){
				$response_data = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
				if( !empty($response_data[0]->disclaimer) ){
					$table = 'cdp_data';
					$data = array('blog_id'=>$this->blog,'data_key'=>'disclaimer','data_value'=>$response_data[0]->disclaimer);
					$format = array('%d','%s','%s');
					$this->wpdb->insert($table,$data,$format);
				}
			}
		}
	}

	function add_cdp_data_dealer(){
		// Checks and creates entries to cdp_data from pre-set companies through an Auto Mall

		$query = 'Select * from cdp_companies where blog_id = "'.$this->blog.'"';
		$dealers = $this->wpdb->get_results($query, ARRAY_A);
		if( !empty($dealers) ){
			foreach($dealers as $dealer){
				$query = 'Select * from cdp_data where blog_id ="'.$this->blog.'" and dealer_id="'.$dealer['dealer_id'].'" and data_key="dealer"';
				$data_check = $this->wpdb->get_results($query, ARRAY_A);
				if( empty($data_check) ){
					$table = 'cdp_data';
					$format = array('%d','%d','%s','%s');
					// Insert Dealer
					$data = array('blog_id'=>$this->blog,'dealer_id'=>$dealer['dealer_id'],'data_key'=>'dealer','data_value'=>'');
					$this->wpdb->insert($table,$data,$format);
				}
			}
		}
	}

	function add_manual_dealers(){
		// Checks and creates entries to cdp_companies for manually added dealers

		$query = 'Select * from cdp_data where blog_id = "'.$this->blog.'" and data_key = "dealer"';
		$dealers = $this->wpdb->get_results($query, ARRAY_A);
		if( !empty($dealers) ){
			foreach($dealers as $dealer){
				$query = 'Select * from cdp_companies where blog_id ="'.$this->blog.'" and dealer_id="'.$dealer['dealer_id'].'" and company_id="'.$this->company_id.'"';
				$data_check = $this->wpdb->get_results($query, ARRAY_A);
				if( empty($data_check) ){
					$api_params = '/api/companies/'.$dealer['dealer_id'];
					$response = $this->http_get_file($api_params);
					$response_code = wp_remote_retrieve_response_code($response);
					if( $response_code == 200 ){
						$this->dealer = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
						if( $this->dealer ){
							$this->populate_company_table();
						}
					}
				}
			}
		}
	}

	function manual_dealer_check($manual_id){

		$results = array();
		if( $manual_id ){
			$query = 'Select * from cdp_data where blog_id = "'.$this->blog.'" and data_key = "dealer" and dealer_id = "'.$manual_id.'"';
			$data_dealer = $this->wpdb->get_results($query, ARRAY_A);
			if( empty($data_dealer) ){
				$api_params = '/api/companies/'.$manual_id;
				$response = $this->http_get_file($api_params);
				$response_code = wp_remote_retrieve_response_code($response);
				if( $response_code == 200 ){
					$results['found'] = 'Company ID added to company. Will import next update.';
				} else {
					$results['error'] = 'Company ID not found. Please enter a different ID or try again.';
				}
			} else {
				$results['error'] = 'Company ID already manually added. Please enter a different ID.';
			}

		} else {
			$results['error'] = 'Company ID not found. Please enter a different ID or try again.';
		}
		return $results;
	}

	function insert_manual_dealer($dealer){
		if( !empty($dealer) ){
			$api_params = '/api/companies/'.$dealer;
			$response = $this->http_get_file($api_params);
			$response_code = wp_remote_retrieve_response_code($response);
			if( $response_code == 200 ){
				$this->dealer = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
				if( $this->dealer ){
					$this->populate_company_table();
				}
			}
		}
	}

	function remove_invalid_vehicles(){

		$query = 'select dealer_id from cdp_companies where blog_id = "'.$this->blog.'" and company_id = "'.$this->company_id.'" ';
		$results = $this->wpdb->get_results($query, ARRAY_A);
		$ids = array();
		foreach( $results as $key=>$value){
			$ids[] = $value['dealer_id'];
		}
		if( !empty($ids) ){
			$query = 'DELETE from cdp_inventory_'.$this->blog.' where blog_id = "'.$this->blog.'" and company_id = "'.$this->company_id.'" and dealer_id not in ('.implode(',',$ids).')';
			$this->wpdb->query($query);
		}

	}

	function init_inventory_clear(){

		$query = 'TRUNCATE cdp_inventory_'.$this->blog;
		$this->wpdb->query($query);
	}

	function init_dealer_refresh(){
		 $query_results = TRUE;
		error_log('CDP Import: Dealer Manual Refresh');
		delete_transient('cdp_company_check');
		$query = 'DELETE from cdp_companies where blog_id="'.$this->blog.'"';
		$sql_results = $this->wpdb->query($query);
		if( $query_results !== FALSE ){
			$this->company_check();
		}
	}

	function init_dealer_call(){
		$api_params = '/api/companies/'.$this->company_id;
		$response = $this->http_get_file($api_params);
		$response_code = wp_remote_retrieve_response_code($response);
		if( $response_code == 200 ){
			$this->dealer = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
			if( $this->dealer ){
				$this->populate_company_table();
				if( isset($this->dealer->automall_ids) ){
					$dealers = $this->dealer->automall_ids;
					foreach( $dealers as $dealer ){
						if( $dealer != $this->company_id){
							$api_params = '/api/companies/'.$dealer;
							$response = $this->http_get_file($api_params);
							$response_code = wp_remote_retrieve_response_code($response);
							if( $response_code == 200 ){
								$this->dealer = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
								$this->populate_company_table();
							}
						}
					}
				}
			}
		}
	}

	function create_data_table(){


		$charset_collate = $this->wpdb->get_charset_collate();
		$table_main = 'cdp_data';

		if( $this->wpdb->get_var("SHOW TABLES LIKE '$table_main'") != $table_main ) {
			error_log('CDP Data table created');
			$columns = "
				data_id bigint(20) NOT NULL AUTO_INCREMENT,
					blog_id int(11) NOT NULL,
				dealer_id int(11) NOT NULL,
				data_key varchar(255) NOT NULL DEFAULT '',
				data_value longtext NOT NULL DEFAULT '',
				UNIQUE KEY (data_id)
			";

			$sql = "CREATE TABLE $table_main ( ".$columns." ) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );
		}
	}

	function create_company_table(){

		$charset_collate = $this->wpdb->get_charset_collate();
		$table_main = 'cdp_companies';

		// Add New Columns
		if( version_compare( $this->database_company, $this->DBcompany_version ) != 0 && $this->wpdb->get_var("SHOW TABLES LIKE '$table_main'") == $table_main ){
			error_log('Company DB Updated: ' . $this->database_company . ' | Current: ' . $this->DBcompany_version);
			/***
				id added 08/31/2018 | DB Version 4.9.180800
			***/
			$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_companies like 'id'");
			if( empty($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_companies ADD COLUMN id bigint(20) NOT NULL AUTO_INCREMENT PRIMARY KEY");
				error_log('Company DB Updated added column id');
			}

			/***
				inventory_api_last_modified added 01/09/2019 | DB Version 4.9.180800
			***/
			$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_companies like 'inventory_api_last_modified'");
			if( empty($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_companies ADD COLUMN inventory_api_last_modified int(11) NOT NULL");
				error_log('Company DB Updated added column inventory_api_last_modified');
			}

			$this->settings['database']['company'] = $this->DBcompany_version;
			$this->update_database_version();

		} else if( $this->wpdb->get_var("SHOW TABLES LIKE '$table_main'") != $table_main ) {
			error_log('Company DB Created: ' . $this->database_company . ' | Current: ' . $this->DBcompany_version);

			$columns = "
				id bigint(20) NOT NULL AUTO_INCREMENT PRIMARY KEY,
				blog_id int(11) NOT NULL,
					company_id int(11) NOT NULL,
					dealer_id int(11) NOT NULL,
				name varchar(255) NOT NULL,
					street varchar(255) NOT NULL,
					city varchar(255) NOT NULL,
				state varchar(255) NOT NULL,
				zip varchar(255) NOT NULL,
				phone varchar(255) NOT NULL,
				country varchar(255) NOT NULL,
				apikeys longtext NOT NULL,
				automalls longtext NOT NULL,
				created date NOT NULL,
				last_updated int(11) NOT NULL,
				inventory_last_updated int(11) NOT NULL,
				inventory_api_last_modified int(11) NOT NULL
			";

			$sql = "CREATE TABLE $table_main ( ".$columns." ) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );

			$this->settings['database']['company'] = $this->DBcompany_version;
			$this->update_database_version();
		}

		/***
			apikeys (AIS related) added 07/29/2020 | DB Version 5.4.200704
		***/
		$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_companies like 'apikeys'");
		if( !empty($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_companies DROP COLUMN apikeys");
				error_log('Company DB Updated removed column apikeys');
				$this->settings['database']['company'] = $this->DBcompany_version;
				$this->update_database_version();
		}
	}

	function populate_company_table(){
		if( $this->dealer ){
			$table_data = array();

			$dealer_id = (isset($this->dealer->id)? $this->dealer->id: 0);
			$name = (isset($this->dealer->name)? $this->dealer->name: '');
			$street = (isset($this->dealer->street)? $this->dealer->street: '');
			$city = (isset($this->dealer->city)? $this->dealer->city: '');
			$state = (isset($this->dealer->state)? $this->dealer->state: '');
			$zip = (isset($this->dealer->zip)? $this->dealer->zip: '');
			$phone = (isset($this->dealer->phone)? $this->dealer->phone: '');
			$country = (isset($this->dealer->country_code)? $this->dealer->country_code: '');
			$automalls = (isset($this->dealer->automall_ids)? $this->dealer->automall_ids: array());
			$created = (isset($this->dealer->created)? date("Y-m-d", strtotime($this->dealer->created) ): '2017-04-01');
			$updated = (isset($this->dealer->updated)? strtotime($this->dealer->updated): current_time( 'timestamp', 0 ));
			$runtime =  0;

			$table_data[] = array(
				'blog_id'=>$this->blog,
				'company_id'=>$this->company_id,
				'dealer_id'=>$dealer_id,
				'name'=>$name,
				'street'=>$street,
				'city'=>$city,
				'state'=>$state,
				'zip'=>$zip,
				'phone'=>$phone,
				'country'=>$country,
				'automalls'=>serialize($automalls),
				'created'=>$created,
				'last_updated'=>$updated,
				'inventory_last_updated'=>$runtime,
				'inventory_api_last_modified'=>0
			);

			if($table_data){
				$this->wpdb->insert('cdp_companies',$table_data[0]);
			}
		}
	}

	function create_inventory_table(){

		$charset_collate = $this->wpdb->get_charset_collate();
		$table_main = 'cdp_inventory_'.$this->blog;
		$table_import = 'cdp_inventory_import_'.$this->blog;

		// Add New Columns
		if( version_compare( $this->database_inventory, $this->DBinventory_version ) != 0 && $this->wpdb->get_var("SHOW TABLES LIKE '$table_main'") == $table_main ){
			error_log('Inventory DB Updated: ' . $this->database_inventory . ' | Current: ' . $this->DBinventory_version);
			/***
				pricesort added 02/12/2018 | DB Version 4.9.180200
			***/
			$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_inventory_import_".$this->blog." like 'pricesort'");
			if( empty($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_inventory_import_".$this->blog." ADD COLUMN pricesort INT DEFAULT 0");
				$this->wpdb->query("ALTER TABLE cdp_inventory_".$this->blog." ADD COLUMN pricesort INT DEFAULT 0");
				error_log('Inventory DB Updated added column pricesort');
			}

			/***
				color_map added 04/17/2018 | DB Version 4.9.180400
			***/
			$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_inventory_import_".$this->blog." like 'color_map'");
			if( empty($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_inventory_import_".$this->blog." ADD COLUMN color_map varchar(255) NOT NULL");
				$this->wpdb->query("ALTER TABLE cdp_inventory_".$this->blog." ADD COLUMN color_map varchar(255) NOT NULL");
				error_log('Inventory DB Updated added column color_map');
			}
			/***
				id data type update 02/26/2019 | DB Version 5.1.190200
			***/
			$exists = $this->wpdb->get_var("SELECT DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = 'cdp_inventory_".$this->blog."' AND COLUMN_NAME = 'id'");
			if( !empty($exists) && $exists == 'mediumint' ){
				$this->wpdb->query("ALTER TABLE cdp_inventory_import_".$this->blog." MODIFY COLUMN id bigint(20) NOT NULL AUTO_INCREMENT");
				$this->wpdb->query("ALTER TABLE cdp_inventory_".$this->blog." MODIFY COLUMN id bigint(20) NOT NULL AUTO_INCREMENT");
				error_log('Inventory DB Updated altered id data type');
			}
			/***
				color_map_int added 04/20/2019 | DB Version 5.1.190400
			***/
			$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_inventory_import_".$this->blog." like 'color_map_int'");
			if( empty($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_inventory_import_".$this->blog." ADD COLUMN color_map_int varchar(255) NOT NULL");
				$this->wpdb->query("ALTER TABLE cdp_inventory_".$this->blog." ADD COLUMN color_map_int varchar(255) NOT NULL");
				error_log('Inventory DB Updated added column color_map_int');
			}
			/***
				carfax_image_url added 08/28/2019 | DB Version 5.2.190800
			***/
			$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_inventory_import_".$this->blog." like 'carfax_image_url'");
			if( empty($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_inventory_import_".$this->blog." ADD COLUMN carfax_image_url varchar(255) NOT NULL");
				$this->wpdb->query("ALTER TABLE cdp_inventory_".$this->blog." ADD COLUMN carfax_image_url varchar(255) NOT NULL");
				error_log('Inventory DB Updated added column carfax_image_url');
			}
			/***
				thumbnail increase varchar added 01/20/2020 | DB Version 5.3.200101
				due to stock image url length
			***/
			$exists = $this->wpdb->get_var("SHOW COLUMNS FROM cdp_inventory_import_".$this->blog." like 'thumbnail'");
			if( isset($exists) ){
				$this->wpdb->query("ALTER TABLE cdp_inventory_import_".$this->blog." MODIFY thumbnail varchar(350) NOT NULL");
				$this->wpdb->query("ALTER TABLE cdp_inventory_".$this->blog." MODIFY thumbnail varchar(350) NOT NULL");
				error_log('Inventory DB Update column thumbnail varchar length');
			}

			$this->settings['database']['inventory'] = $this->DBinventory_version;
			$this->update_database_version();
		} else if( $this->wpdb->get_var("SHOW TABLES LIKE '$table_main'") != $table_main ){
			error_log('Inventory DB Created: ' . $this->database_inventory . ' | Current: ' . $this->DBinventory_version);

			$columns = "
				id bigint(20) NOT NULL AUTO_INCREMENT,
				blog_id int(11) NOT NULL DEFAULT '1',
				vms_id int(11) NOT NULL,
					company_id int(11) NOT NULL,
					dealer_id int(11) NOT NULL,
				dealer varchar(255) NOT NULL,
					stock varchar(255) NOT NULL,
					vin varchar(255) NOT NULL,
				saleclass varchar(255) NOT NULL,
				certified tinyint(1) NOT NULL DEFAULT '0',
					make varchar(255) NOT NULL,
					model varchar(255) NOT NULL,
				model_code varchar(255) NOT NULL,
					trim varchar(255) NOT NULL,
					year int(11) NOT NULL,
				askingprice decimal(18,0) NOT NULL,
				saleprice decimal(18,0) NOT NULL,
				retailprice decimal(18,0) NOT NULL,
				msrp decimal(18,0) NOT NULL,
				sortprice decimal(18,0) NOT NULL,
				pricesort int(11) NOT NULL DEFAULT '0',
				sale_expire varchar(255) NOT NULL,
				headline varchar(512) NOT NULL,
				description text NOT NULL,
				exterior varchar(255) NOT NULL,
				interior varchar(255) NOT NULL,
				color_map varchar(255) NOT NULL,
				color_map_int varchar(255) NOT NULL,
				body varchar(255) NOT NULL,
				vehicleclass varchar(255) NOT NULL,
				tags varchar(255) NOT NULL,
				drivetrain varchar(255) NOT NULL,
				doors varchar(10) NOT NULL,
				engine varchar(255) NOT NULL,
				transmission varchar(255) NOT NULL,
				fuel_type varchar(255) NOT NULL,
				highway varchar(255) NOT NULL,
				city varchar(255) NOT NULL,
				odometer int(11) NOT NULL,
				carfax varchar(255) NOT NULL,
				carfax_image_url varchar(255) NOT NULL,
				autocheck varchar(255) NOT NULL,
				video varchar(255) NOT NULL,
				acode varchar(255) NOT NULL,
				thumbnail varchar(350) NOT NULL,
				geo_state varchar(255) NOT NULL,
				geo_city varchar (255) NOT NULL,
				geo_zip varchar (255) NOT NULL,
				location varchar (255) NOT NULL,
				phone varchar (100) NOT NULL,
				created datetime NOT NULL,
				updated datetime NOT NULL,
				sort int(11) NOT NULL DEFAULT '0',
				extra text NOT NULL,
				UNIQUE KEY id (id)
			";
			$sql = "CREATE TABLE $table_main ( ".$columns." ) $charset_collate;";
			$sql2 = "CREATE TABLE $table_import ( ".$columns." ) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );
			dbDelta( $sql2 );

			$this->settings['database']['inventory'] = $this->DBinventory_version;
			$this->update_database_version();
		}
	}

	function remove_dealer_data_clear_handler(){
		if($this->company_id){
			// Remove Data from CDP Data
			error_log('Removing all dealers from CDP Data Table');
			$where = array('blog_id'=>$this->blog,'data_key'=>'dealer');
			$where_format = array('%d','%s');
			$this->wpdb->delete('cdp_data',$where,$where_format);

			// Remove Data from CDP Companies
			error_log('Removing all dealers from CDP Companies Table');
			$where = array('blog_id'=>$this->blog);
			$where_format = array('%d');
			$this->wpdb->delete('cdp_companies',$where,$where_format);

			// Remove Data from CDP Inventory
			error_log('Removing all dealers from CDP Inventory '.$this->blog.' Table');
			$where = array('blog_id'=>$this->blog);
			$where_format = array('%d');
			$this->wpdb->delete('cdp_inventory_'.$this->blog,$where,$where_format);
		}
	}

	function remove_dealer_data_handler($remove_dealer=0,$new_dealer=0){
		if($remove_dealer){
			// Remove Data from CDP Data
			error_log('Removing dealer '.$remove_dealer.' from CDP Data Table');
			$where = array('blog_id'=>$this->blog,'dealer_id'=>$remove_dealer,'data_key'=>'dealer');
			$where_format = array('%d','%d','%s');
			$this->wpdb->delete('cdp_data',$where,$where_format);

			// Remove Data from CDP Companies
			error_log('Removing dealer '.$remove_dealer.' from CDP Companies Table');
			$where = array('blog_id'=>$this->blog,'dealer_id'=>$remove_dealer);
			$where_format = array('%d','%d');
			$this->wpdb->delete('cdp_companies',$where,$where_format);

			// Remove Data from CDP Inventory
			error_log('Removing dealer '.$remove_dealer.' from CDP Inventory '.$this->blog.' Table');
			$where = array('blog_id'=>$this->blog,'dealer_id'=>$remove_dealer);
			$where_format = array('%d','%d','%d');
			$this->wpdb->delete('cdp_inventory_'.$this->blog,$where,$where_format);

			if( $new_dealer ){
				// Update any previous dealer to new company id
				error_log('Update company dealers to new company id '.$new_dealer);
				$data = array('company_id'=>$new_dealer);
				$where = array('blog_id'=>$this->blog,'company_id'=>$remove_dealer);
				$format = array('%d');
				$where_format = array('%d','%d');
				$this->wpdb->update('cdp_companies', $data, $where, $format, $where_format);

				// Update any previous dealer to new company id
				error_log('Update inventory dealers to new company id '.$new_dealer);
				$data = array('company_id'=>$new_dealer);
				$where = array('blog_id'=>$this->blog,'company_id'=>$remove_dealer);
				$format = array('%d');
				$where_format = array('%d','%d');
				$this->wpdb->update('cdp_inventory_'.$this->blog, $data, $where, $format, $where_format);
			}

		}
	}

	function insert_dealer_data_handler($new_dealer=0){
		if( $new_dealer ){
			$query_prepare = 'select * from cdp_data where blog_id = %d and data_key = %s and dealer_id = %d';
			$query_params_prepare = array($this->blog,'dealer',$new_dealer);
			$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
			$data_dealer = $this->wpdb->get_results($query, ARRAY_A);
			// This should be empty so duplicates are not added.
			if( empty($data_dealer) ){
				$api_params = '/api/companies/'.$new_dealer;
				$response = $this->http_get_file($api_params);
				$response_code = wp_remote_retrieve_response_code($response);
				if( $response_code == 200 ){
					error_log('New Dealer ('.$new_dealer.') added to CDP Data table');
					$data = array('blog_id'=>$this->blog,'dealer_id'=>$new_dealer,'data_key'=>'dealer','data_value'=>$new_dealer);
					$format = array('%d','%d','%s','%s');
					$this->wpdb->insert('cdp_data',$data,$format);
					delete_transient('cdp_company_processing_'.$this->blog);
				}
			}
		}
	}

	function init_company_handler($automall_run = FALSE){
		$this->set_option_variables();
		if($this->company_id){
			if( !get_transient('cdp_company_processing_'.$this->blog) ){
				set_transient('cdp_company_processing_'.$this->blog, 'Running', 60 );
				// Start by checking/adding additional dealers
				$query_prepare = 'select * from cdp_data where blog_id = %d and data_key = %s';
				$query_params_prepare = array($this->blog,'dealer');
				$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
				$data_dealers = $this->wpdb->get_results($query, ARRAY_A);
				if(!empty($data_dealers)){
					// Loop through cdp_data dealers
					foreach($data_dealers as $key=>$data_dealer){
						$query_prepare = 'select * from cdp_companies where blog_id = %d and dealer_id = %d';
						$query_params_prepare = array($this->blog,$data_dealer['dealer_id']);
						$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
						$dealer_check = $this->wpdb->get_results($query, ARRAY_A);
						// Add Dealer to companies table if not found
						if( empty($dealer_check) ){
							error_log('Dealer '.$data_dealer['dealer_id'].' missing from company | Adding to company table');
							$data = array('blog_id'=>$this->blog,'company_id'=>$this->company_id,'dealer_id'=>$data_dealer['dealer_id'],'last_updated'=>0);
							$format = array('%d','%d','%d','%d');
							$this->wpdb->insert('cdp_companies',$data,$format);
						}
					}
					// Now loop through all companie entries and update if needed
					$query_prepare = 'select * from cdp_companies where blog_id = %d';
					$query_params_prepare = array($this->blog);
					$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
					$dealers = $this->wpdb->get_results($query, ARRAY_A);
					if($dealers){
						foreach($dealers as $key=>$dealer){
							$api_params = '/api/companies/'.$dealer['dealer_id'];
							$response = $this->http_get_file($api_params);
							$response_code = wp_remote_retrieve_response_code($response);
							if( $response_code == 200 ){
								$response_data = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
								// Update Company entry if new or update needed
								if( $dealer['last_updated'] != strtotime($response_data->updated) || $dealer['last_updated'] == 0 ){
									error_log('Company '.$dealer['dealer_id'].' beeing updated');
									if( isset($response_data->seo->city) && !empty($response_data->seo->city) ){
										$dealer_city = str_replace("'", "", $response_data->seo->city);
									} else {
										$dealer_city = str_replace("'", "", $response_data->city);
									}
									if( isset($response_data->seo->state) && !empty($response_data->seo->state) ){
										$dealer_state = str_replace("'", "", $response_data->seo->state);
									} else {
										$dealer_state = str_replace("'", "", $response_data->state);
									}
									$dealer_name = str_replace("'", "", $response_data->name);
									$dealer_street = str_replace("'", "", $response_data->street);
									$automalls = serialize((isset($response_data->automall_ids)? $response_data->automall_ids: array()));
									$created = (isset($response_data->created)? date("Y-m-d", strtotime($response_data->created) ): '2017-04-01');
									$updated = (isset($response_data->updated)? strtotime($response_data->updated): 0);
									$inv_updated = (isset($dealer['inventory_last_updated'])? $dealer['inventory_last_updated']: 0);
									$inv_api_modified = (isset($dealer['inventory_api_last_modified'])? $dealer['inventory_api_last_modified']: 0);

									$data = array('company_id'=>$this->company_id,'name'=>$dealer_name, 'street'=>$dealer_street, 'city'=>$dealer_city, 'state'=>$dealer_state, 'zip'=>$response_data->zip, 'phone'=>$response_data->phone, 'country'=>$response_data->country_code, 'automalls'=>$automalls, 'created'=>$created, 'last_updated'=>$updated, 'inventory_last_updated'=>$inv_updated, 'inventory_api_last_modified'=>$inv_api_modified );
									$where = array('id'=>$dealer['id']);
									$format = array('%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d','%d');
									$where_format = array('%d');
									$this->wpdb->update('cdp_companies',$data,$where,$format,$where_format);
									// Update Inventory with new data
									if ( ! wp_next_scheduled( 'cdp_cron_inventory_update' ) ) {
										error_log('Company Inventory update backup. No Cron found');
										$this->init_inventory_handler($dealer['dealer_id']);
									}

								}
							}
							// Remove dealer from companies if not found in cdp data
							$query_prepare = 'select * from cdp_data where blog_id = %d and dealer_id = %d and data_key = %s';
							$query_params_prepare = array($this->blog,$dealer['dealer_id'],'dealer');
							$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
							$dealer_remove_check = $this->wpdb->get_results($query, ARRAY_A);
							if( empty($dealer_remove_check) ){
								error_log('Dealer '.$dealer['dealer_id'].' being removed from companies as no longer found in CDP Data');
								$where = array('id'=>$dealer['id']);
								$where_format = array('%d');
								$this->wpdb->delete('cdp_companies',$where,$where_format);
							}
						}
						// Run Cron Job once if able
						if ( wp_next_scheduled( 'cdp_cron_inventory_update' ) ) {
							error_log('Company Inventory update cron init.');
							wp_schedule_single_event(time(), 'cdp_cron_inventory_update');
						}
					}
					// Now check if Primary Company has an automall and clean up new/remove dealers
					if( !$automall_run ){
						$new_automall_found = FALSE;
						$query_prepare = 'select * from cdp_companies where blog_id = %d and company_id = %d and dealer_id = %d';
						$query_params_prepare = array($this->blog,$this->company_id,$this->company_id);
						$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
						$dealer = $this->wpdb->get_results($query, ARRAY_A);
						if( $dealer ){
							$automalls = unserialize($dealer[0]['automalls']);
							if( !empty($automalls) ){
								// Delete Dealers from CDP Data and Company if ID removed from automall
								$query_prepare = 'select * from cdp_data where blog_id = %d and data_key = %s and data_value = %s';
								$query_params_prepare = array($this->blog,'dealer','automall');
								$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
								$data_automalls = $this->wpdb->get_results($query, ARRAY_A);
								if( $data_automalls ){
									foreach($data_automalls as $key=>$data_automall){
										if( !in_array($data_automall['dealer_id'], $automalls) ){
											error_log('Automall Dealer ('.$data_automall['dealer_id'].') no longer found in Company Automall');
											// Remove Company Value
											$where = array('blog_id'=>$this->blog,'company_id'=>$this->company_id,'dealer_id'=>$data_automall['dealer_id']);
											$where_format = array('%d','%d','%d');
											$this->wpdb->delete('cdp_companies',$where,$where_format);
											// Remove Data Valut
											$where = array('data_id'=>$data_automall['data_id']);
											$where_format = array('%d');
											$this->wpdb->delete('cdp_data',$where,$where_format);
										}
									}
								}
								// Add/Update automall dealers in CDP Data
								foreach($automalls as $key=>$automall){
									// Check if automall dealer is already added to CDP data and update or add
									$query_prepare = 'select * from cdp_data where blog_id = %d and data_key = %s and dealer_id = %d';
									$query_params_prepare = array($this->blog,'dealer',$automall);
									$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
									$data_check = $this->wpdb->get_results($query, ARRAY_A);
									if( empty($data_check) ){
										// Add Dealer to CDP Data
										$data = array('blog_id'=>$this->blog,'dealer_id'=>$automall,'data_key'=>'dealer','data_value'=>'automall');
										$data_format = array('%d','%d','%s','%s');
										$this->wpdb->insert('cdp_data',$data,$data_format);
										$new_automall_found = TRUE;
									}  else {
										// Update Dealer in CDP Data make sure data value is set to automall
										//** This could be due to a dealer that was manually added in the plugin being added as an automall dealer in VMS
										$data = array('data_value'=>'automall'); $data_format = array('%s');
										$where = array('data_id'=>$data_check[0]['data_id']); $where_format = array('%d');
										$this->wpdb->update('cdp_data',$data,$where,$data_format,$where_format);
									}
								}
								// Re-Run company handler if new a new automall company was added but automall check.
								if( $new_automall_found ){
									delete_transient('cdp_company_processing_'.$this->blog);
									$this->init_company_handler(TRUE);
								}
							} else {
								error_log('Company ID does not contain any automall data');
							}
						}
					}
				}
			}
		}
	}

	function init_inventory_auto_update_handler(){
		if( !get_transient('cdp_auto_dealer_processing_'.$this->blog) ){
			set_transient('cdp_auto_dealer_processing_'.$this->blog, 'Running', 60 );
			$query_prepare = 'select * from cdp_companies where blog_id = %d';
			$query_params_prepare = array($this->blog);
			$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
			$dealers = $this->wpdb->get_results($query, ARRAY_A);
			foreach( $dealers as $key=>$dealer ){
				$automall_check = unserialize($dealer['automalls']);
				if( empty($automall_check) ){
					$inv_init = TRUE;
					error_log('Checking Auto Update for Dealer - '.$dealer['dealer_id']);
					// Get API Last Modified time and Check Response Code
					$api_params = '/'.$dealer['dealer_id'].'/inventory/vehicles.json?page=1&per_page=50&photo_view=1';
					$response = $this->http_get_file_headers($api_params);
					$response_code = wp_remote_retrieve_response_code( $response );
					if( $response_code == 200 ){
						$last_modified = isset($response['headers']['last-modified']) ? strtotime($response['headers']['last-modified']): 0;
						if( $last_modified > 0 ){
							// Run Inventory if either time stamps are zero
							if( $inv_init && ($dealer['inventory_last_updated'] == 0 || $dealer['inventory_api_last_modified'] == 0) ){
								$inv_init = FALSE;
								error_log('CDP Initial Inventory auto update run | Dealer - '.$dealer['dealer_id'].' | Time - '.current_time('timestamp',0) );
								$this->init_inventory_handler($dealer['dealer_id']);
							}
							// Run Inventory if API Last Modified Stamps don't match
							if( $inv_init && $dealer['inventory_api_last_modified'] <> $last_modified ){
								$inv_init = FALSE;
								error_log('CDP API Last Modified change | Dealer - '.$dealer['dealer_id'].' | Time - '.current_time('timestamp',0) . ' | Previous API Last Modified - '.$dealer['inventory_api_last_modified']. ' | Current API Last Modified - '.$last_modified);
								$this->init_inventory_handler($dealer['dealer_id']);
							}
							// Run Inventory if Inventory Table is Empty for Dealer Inventory
							$query_prepare = 'select count(*) Inv_Total from cdp_inventory_'.$this->blog.' where blog_id = %d and dealer_id = %d';
							$query_params_prepare = array($this->blog,$dealer['dealer_id']);
							$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
							$vehicle_count = $this->wpdb->get_results($query, ARRAY_A);
							if( $inv_init && $vehicle_count[0]['Inv_Total'] == 0 ){
								$inv_init = FALSE;
								error_log('CDP Vehicle Count 0 | Dealer - '.$dealer['dealer_id'].' | Time - '.current_time('timestamp',0) );
								$this->init_inventory_handler($dealer['dealer_id']);
							}
						}
					} else {
							error_log('CDP Auto Update check failed | '.$dealer['dealer_id'].' | response code: '.$response_code.'| url: '.$this->api_call.$api_params );
					}

				} else {
					error_log('Auto Update has dealer '.$dealer['dealer_id'].' as a main automall account.  Will not update.');
				}
			}
			delete_transient('cdp_auto_dealer_processing_'.$this->blog);
		} else {
			//Do Nothing
			error_log('CDP Auto Updater is already running');
		}
	}

	function init_inventory_handler($dealer=0,$purge=FALSE){
		$singleDealer = '';
		if( $dealer !== 0 ){
			$singleDealer = ' AND dealer_id = %d';
		}
		if($purge === TRUE){
			$this->runPurge = TRUE;
		}
		$query_prepare = 'select * from cdp_companies where blog_id = %d'.$singleDealer;
		if( empty($singleDealer) ){
			$query_params_prepare = array($this->blog);
		} else {
			$query_params_prepare = array($this->blog, $dealer);
		}
		$query = $this->wpdb->prepare($query_prepare, $query_params_prepare);
		$dealers = $this->wpdb->get_results($query, ARRAY_A);
		foreach( $dealers as $key=>$dealer ){
			$automall_check = unserialize($dealer['automalls']);
			if( empty($automall_check) ){
				//Allow the process to run for 2 minutes needed for large automalls.
				set_time_limit(120);
				$this->dealer = $dealer;
				$api_params = '/'.$dealer['dealer_id'].'/inventory/vehicles.json?page=1&per_page=50&photo_view=1';
				$response = $this->http_get_file($api_params);
				$response_code = wp_remote_retrieve_response_code( $response );
				// Confirm response_code returns 200 before removing inventory and re updating
				if(  $response_code == 200 ) {
					// Set Last Modified value from API
					$last_modified = isset($response['headers']['last-modified']) ? strtotime($response['headers']['last-modified']): 0;
					// Clear Import Table
					$this->clear_import_table();
					// Create page count variable
					$response_data = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
					$this->total_count = is_array($response_data) && isset($response_data[0]) && count($response_data) > 0? $response_data[0]->pagination->total: 0;
					$this->page_count = ( $this->total_count > 0? $this->total_count + 1: 0 );
					// Populate Import Table
					$result = $this->inventory_call_data();
					if ( $result !== FALSE ){
						// Remove Current Dealer Vehicles from Live Inventory Update
						$this->wpdb->delete('cdp_inventory_'.$this->blog,array('blog_id'=>$this->blog, 'dealer_id'=>$dealer['dealer_id']), array('%d','%d') );
						// Populate Live Table
						$this->update_live_table();
						// Update Company Import Times
						$update_values = array('inventory_last_updated'=>current_time('timestamp', 0),'inventory_api_last_modified'=>$last_modified);
						$where_values = array('blog_id'=>$this->blog,'dealer_id'=>$dealer['dealer_id']);
						$format = array('%d','%d');
						$this->wpdb->update('cdp_companies',$update_values,$where_values,$format,$format);
						// Update Sort Values
						if( $this->sortData ){
							cdp_plugin::$db_caller->cdp_admin_update_sort($this->sortData);
						}
					} else {
						error_log('CDP API Inventory Update Handler failed, due to failed api calls.');
					}
				} else {
					error_log('CDP API Inventory Update Handler check failed | response code '.$response_code);
				}
			} else {
				error_log('Manual Update has dealer '.$dealer['dealer_id'].' as a main automall account. Will not update.');
			}
		}
	}

	function http_get_file_purge($params){

		// Get site URL and pass as referer to API call
		$url = $this->api_call.$params;
		$siteURL = site_url();
		$request_parameters['headers']['Referer'] = $siteURL;
		$request_parameters['method']='PURGE';
		$response = wp_remote_request( $url , $request_parameters );
		error_log('API Purge ran for URL: '.$url);
		//return $response;
	}

	function http_get_file($params){
		// Get site URL and pass as referer to API call
		$url = $this->api_call.$params;
		$siteURL = site_url();
		$request_parameters['headers']['Referer'] = $siteURL;
		$response = wp_remote_request( $url , $request_parameters );
		return $response;
	}

	function http_get_file_headers($params){
		// Get site URL and pass as referer to API call
		$url = $this->api_call.$params;
		$siteURL = site_url();
		$request_parameters['headers']['Referer'] = $siteURL;
		$response = wp_remote_head( $url , $request_parameters );
		return $response;
	}

	function clear_import_table(){
		$this->wpdb->query('TRUNCATE cdp_inventory_import_'.$this->blog);
	}

	function inventory_call_data(){
		$this->vehicle_data = $vehicle_array = array(); $page = 1; $restraint = 50;
		do{
			$i = 0; $temp_array = $vehicle_array = array();
			do{
				if( $page <= $this->page_count ){
					$temp_array[] = $this->build_inventory_request_list($page);
					$page++;$i++;
				} else {
					break 1;
				}
			}while( $i < $restraint);
			foreach($temp_array as $key=>$api_params){
				if( $this->runPurge ){
					$this->http_get_file_purge($api_params);
				}
				$url = $this->api_call.$api_params;
				$response = $this->http_get_file($api_params);
				$response_code = wp_remote_retrieve_response_code( $response );
				$response_message = wp_remote_retrieve_response_message( $response );
				error_log('CDP API Import Page | code: '.$response_code. ' | message: '.$response_message.' | request: '.$url);
				if( $response_code == 200 ){
					//wp_kses_data breaks json format when escape characters are removed
					//$response_data = isset($response['body']) ?json_decode(wp_kses_data($response['body'])): array();
					$response_data = isset($response['body']) ?json_decode($response['body']): array();
					if( !empty($response_data) ){
						$vehicles = $this->scrub_vehicle($response_data);
						$this->vehicle_data[] = $vehicles;
						$this->build_inventory_table_data();
						unset($this->vehicle_data);
						$this->populate_import_table();
					} else {
						error_log('Response Body Empty for URL: '.$url);
					}
				} else {
					error_log('CDP API Import Page failed | request: '.$url);
					sleep(3);
					$response = $this->http_get_file($url);
					$response_code = wp_remote_retrieve_response_code( $response );
					$response_message = wp_remote_retrieve_response_message( $response );
					error_log('CDP API Import Page 2nd Attempt | code: '.$response_code. ' | message: '.$response_message.' | request: '.$url);
					if( $response_code == 200 ){
						$response_data = isset($response['body']) ?json_decode($response['body']): array();
						if( !empty($response_data) ){
							$vehicles = $this->scrub_vehicle($response_data);
							$this->vehicle_data[] = $vehicles;
							$this->build_inventory_table_data();
							unset($this->vehicle_data);
							$this->populate_import_table();
						} else {
							error_log('CPD API 2nd Response Body Empty for URL: '.$url);
						}
					} else {
						error_log('CDP API Import Page 2nd Attempt failed | request: '.$url);
						error_log('CDP API Failed Import | Stopping Import');
						//exit out of loop if second call fails, and not import data
						return FALSE;
						exit;
					}
				}
			}
		}while( $page < $this->page_count);
	}

	function build_inventory_request_list($page){
		$params = '/'.$this->dealer['dealer_id'].'/inventory/vehicles.json?page='.$page.'&per_page='.$this->per_page_limit.'&photo_view=1';
		$url = $this->api_call.$params;
		error_log('CDP_Import build_inventory_request_list | URL: '.$url);
		return $params;
	}

	function scrub_vehicle($data){
		$result = array();
		$keys = array( 'stock_number','vin','saleclass','make','model_name','trim','year','contact_info','exterior_color','interior_color','tags','body_style','vehicleclass', 'id' ,'drive_train', 'prices', 'video_url', 'doors', 'engine', 'transmission', 'odometer', 'carfax', 'carfax_image_url', 'photos', 'headline', 'description', 'certified_bool', 'created_at', 'ads_acode', 'updated_at', 'auto_check_url','vehicle_location','fuel_type','model_code','fuel_economy' );
		if( $data ){
			foreach($data as $i=>$values){
				foreach($keys as $key){
					if( is_array($values) ){
						$result[$i][$key] = (isset($values[$key])? $values[$key]: '');
					} else {
						$result[$i][$key] = (isset($values->$key)? $values->$key: '');
					}
				}
			}
		}
		return $result;
	}

	function init_inventory_color_update(){

		$vehicles = cdp_plugin::$db_caller->cdp_inventory_special_request('id,exterior');
		if( $vehicles ){
			foreach( $vehicles as $key=>$values ){
				$color = $this->exterior_color_mapping($values['exterior']);
				$updated = $this->wpdb->update('cdp_inventory_'.$this->blog,array('color_map'=>$color), array('id'=>$values['id']), array('%s'), array('%d') );
				if( $updated === false ){
					error_log('CDP Color Map error on ID '.$values['id']);
				}
			}
		}
		// Copies logic for Exterior Color Map but for Interior
		$vehicles = cdp_plugin::$db_caller->cdp_inventory_special_request('id,interior');
		if( $vehicles ){
			foreach( $vehicles as $key=>$values ){
				$color = $this->exterior_color_mapping($values['interior']);
				$updated = $this->wpdb->update('cdp_inventory_'.$this->blog,array('color_map_int'=>$color), array('id'=>$values['id']), array('%s'), array('%d') );
				if( $updated === false ){
					error_log('CDP Color Map error on ID '.$values['id']);
				}
			}
		}
	}

	function exterior_color_mapping($color){
		$default_color = ( isset($this->settings['vehicle_management_system']['color_mapping']['default_text'])? $this->settings['vehicle_management_system']['color_mapping']['default_text']: 'Unknown');
		if( empty($color) ){
			return $default_color;
		} else {
			$color_map = ''; $color = strtolower($color);
			foreach( $this->settings['vehicle_management_system']['color_mapping']['exterior'] as $key=>$values ){
				if( empty($color_map) ){
					$color_search = explode( ',', $values);
					foreach( $color_search as $needle ){
						$needle = trim($needle);
						$pos = strpos($color, strtolower($needle) );
						if( $pos !== false ){
							$color_map = ucfirst($key);
							break;
						}
					}
				} else {
					break;
				}
			}
			if( empty($color_map) ){
				return $default_color;
			} else {
				return $color_map;
			}
		}
	}

	function build_inventory_table_data(){
		$this->table_data = array();
		if( !empty($this->vehicle_data) ){
			foreach( $this->vehicle_data as $page=>$list ){
				foreach( $list as $vehicle ){
					if( !isset($vehicle['sold_on']) ){
						$id = (isset($vehicle['id'])? $vehicle['id']: 0);
						$stock = (isset($vehicle['stock_number'])? $vehicle['stock_number']: '');
						$vin = (isset($vehicle['vin'])? $vehicle['vin']: '');
						$saleclass = (isset($vehicle['saleclass'])? strtolower($vehicle['saleclass']): '');
						$make = (isset($vehicle['make'])? $vehicle['make']: '');
						$model = (isset($vehicle['model_name'])? $vehicle['model_name']: '');
						$model_code = (isset($vehicle['model_code'])? $vehicle['model_code']: '');
						$trim = (isset($vehicle['trim'])? $vehicle['trim']: '');
						$year = (isset($vehicle['year'])? $vehicle['year']: '');
						$exterior = (isset($vehicle['exterior_color'])? $vehicle['exterior_color']: '');
						$interior = (isset($vehicle['interior_color'])? $vehicle['interior_color']: '');
						$color_map = '';
						$body = (isset($vehicle['body_style'])? $vehicle['body_style']: '');
						$vehicleclass = (isset($vehicle['vehicleclass'])? $vehicle['vehicleclass']: '');
						$tags = (isset($vehicle['tags']) && !empty($vehicle['tags'])? implode(',', $vehicle['tags']): '');
						if( is_array($vehicle['contact_info']) ){
							$dealer = (isset($vehicle['contact_info']['company_id'])? $vehicle['contact_info']['company_id']: '');
						} else {
							$dealer = (isset($vehicle['contact_info']->company_id)? $vehicle['contact_info']->company_id: '');
						}

						$prices = (isset($vehicle['prices'])? $vehicle['prices']: array());
						$drivetrain = (isset($vehicle['drive_train'])? $vehicle['drive_train']: '');
						$video = (isset($vehicle['video_url'])? $vehicle['video_url']: '');
						$doors = (isset($vehicle['doors'])? $vehicle['doors']: '');
						$engine = (isset($vehicle['engine'])? $vehicle['engine']: '');
						$transmission = (isset($vehicle['transmission'])? $vehicle['transmission']: '');
						$fuel = (isset($vehicle['fuel_type']) ? ($vehicle['fuel_type'] == 'N/A' ? '':$vehicle['fuel_type']):'');
						$fuel_economy = (isset($vehicle['fuel_economy'])? $vehicle['fuel_economy']: array());
						$odometer = (isset($vehicle['odometer'])? $vehicle['odometer']: '');
						if( is_array($vehicle['carfax']) ){
							$carfax = (isset($vehicle['carfax']['url'])? $vehicle['carfax']['url']: '');
							$carfax_image_url = (isset($vehicle['carfax']['image_url'])? $vehicle['carfax']['image_url']: '');
						} else {
							$carfax = (isset($vehicle['carfax']->url)? $vehicle['carfax']->url: '');
							$carfax_image_url = (isset($vehicle['carfax']->image_url)? $vehicle['carfax']->image_url: '');
						}
						$autocheck = (isset($vehicle['auto_check_url'])? $vehicle['auto_check_url']: '');
						$photos = (isset($vehicle['photos'])? $vehicle['photos']: array());
						$headline = (isset($vehicle['headline'])? substr($vehicle['headline'], 0, 512): '');
						$description = (isset($vehicle['description'])? $vehicle['description']: '');
						$certified = (isset($vehicle['certified_bool']) && $vehicle['certified_bool'] == true? 1: 0);
						$created = (isset($vehicle['created_at'])? $vehicle['created_at']: '');
						$updated = (isset($vehicle['updated_at'])? $vehicle['updated_at']: '');
						$acode = (isset($vehicle['ads_acode'])? $vehicle['ads_acode']: '');
						$location = (isset($vehicle['vehicle_location'])? $vehicle['vehicle_location']: '');
						$blog_id = get_current_blog_id();
						// Find sortprice lowest value
						if( is_array($prices) ){
							if( isset($prices['sale_price']) ){
								$sortprice = ( $prices['sale_price'] !== 0 && $prices['sale_price'] < $prices['asking_price']? $prices['sale_price']: $prices['asking_price']);
							}else if( isset($prices['asking_price']) ){
								$sortprice = $prices['asking_price'];
							}else{
								$sortprice = 0;
							}
						} else {
							if( isset($prices->sale_price) ){
								$sortprice = ( $prices->sale_price !== 0 && $prices->sale_price < $prices->asking_price? $prices->sale_price: $prices->asking_price);
							}else if( isset($prices->asking_price) ){
								$sortprice = $prices->asking_pricek;
							}else{
								$sortprice = 0;
							}
						}

						// Edge Case
						$extra = '';
						$extra .= (strtolower($vehicleclass) == 'sport utility'? 'SUV ': '');
						$extra .= (strtolower($make) == 'chevrolet'? 'Chevy ': '');
						$extra .= (strtolower($model) == 'f-150'? 'F150 ': '');

						if( is_array($prices) ){
							$askingprice = (isset($prices['asking_price'])? $prices['asking_price']: 0);
							$saleprice = (isset($prices['sale_price'])? $prices['sale_price']: 0);
							$retailprice = (isset($prices['retail_price'])? $prices['retail_price']: 0);
							$msrp = (isset($prices['msrp'])? $prices['msrp']: 0);
							$sale_expire = (isset($prices['sale_expire'])? $prices['sale_expire']: 0);
						} else {
							$askingprice = (isset($prices->asking_price)? $prices->asking_price: 0);
							$saleprice = (isset($prices->sale_price)? $prices->sale_price: 0);
							$retailprice = (isset($prices->retail_price)? $prices->retail_price: 0);
							$msrp = (isset($prices->msrp)? $prices->msrp: 0);
							$sale_expire = (isset($prices->sale_expire)? $prices->sale_expire: 0);
						}
						if( is_array($fuel_economy) ){
							$highway = (isset($fuel_economy['highway'])? $fuel_economy['highway']: 0);
							$city = (isset($fuel_economy['city'])? $fuel_economy['city']: 0);
						} else {
							$highway = (isset($fuel_economy->highway)? $fuel_economy->highway: 0);
							$city = (isset($fuel_economy->city)? $fuel_economy->city: 0);
						}
						if( is_array($photos[0]) ){
							$thumbnail = (isset($photos[0]['small'])? $photos[0]['small']: '');
						} else {
							$thumbnail = (isset($photos[0]->small)? $photos[0]->small: '');
						}

						$this->table_data[] = array(
							'blog_id'=>$blog_id,
							'vms_id'=>$id,
							'company_id'=>$this->company_id,
							'dealer_id'=>$dealer,
							'dealer'=>(isset($this->dealer['name'])? $this->dealer['name']: ''),
							'stock'=>$stock,
							'vin'=>$vin,
							'saleclass'=>$saleclass,
							'certified'=>$certified,
							'make'=>$make,
							'model'=>$model,
							'model_code'=>$model_code,
							'trim'=>$trim,
							'year'=>$year,
							'askingprice'=>$askingprice,
							'saleprice'=>$saleprice,
							'retailprice'=>$retailprice,
							'msrp'=>$msrp,
							'sortprice'=>$sortprice,
							'pricesort'=>0,
							'sale_expire'=>$sale_expire,
							'headline'=>$headline,
							'description'=>$description,
							'exterior'=>$exterior,
							'interior'=>$interior,
							'color_map'=>'', // Gets set on own update
							'body'=>$body,
							'vehicleclass'=>$vehicleclass,
							'tags'=>(empty($tags)? '': $tags),
							'drivetrain'=>$drivetrain,
							'doors'=>$doors,
							'engine'=>$engine,
							'transmission'=>$transmission,
							'fuel_type'=>$fuel,
							'highway'=>$highway,
							'city'=>$city,
							'odometer'=>$odometer,
							'carfax'=>$carfax,
							'carfax_image_url'=>$carfax_image_url,
							'autocheck'=>$autocheck,
							'video'=>$video,
							'acode'=>$acode,
							'thumbnail'=>$thumbnail,
							'geo_state'=>(isset($this->dealer['state'])? $this->dealer['state']: ''),
							'geo_city'=>(isset($this->dealer['city'])? $this->dealer['city']: ''),
							'geo_zip'=>(isset($this->dealer['zip'])? $this->dealer['zip']: ''),
							'location'=>$location,
							'phone'=>(isset($this->dealer['phone'])? $this->dealer['phone']: ''),
							'created'=>$created,
							'updated'=>$updated,
							'sort'=>0,
							'extra'=>$extra
						);
					}
				}
			}
		}
	}

	function populate_import_table(){

		if($this->table_data){
			$count = count($this->table_data);
			$i = 0;
			do{
				$this->wpdb->insert('cdp_inventory_import_'.$this->blog.'',$this->table_data[$i]);
				$i++;
			}while( $i < $count );
			unset($this->table_data);
		}
	}

	function get_custom_import_symbol($key){
		$result = '';
		switch($key){
			case 0;
				$result = ' IN ';
				break;
			case 1;
				$result = ' NOT IN ';
				break;
			case 2;
				$result = ' > ';
				break;
			case 3;
				$result = ' < ';
				break;
		}
		return $result;
	}

	function build_custom_import_query($import){
		$custom_string = $search = ''; $clean_data = array();
		if( !empty($import) ){
			$custom = unserialize($import);
			// Build custom saleclass
			if( !empty($custom['saleclass']) ){
				if($custom['saleclass'] == 'certified'){
					$custom_string .= ' AND saleclass = "used" AND certified = 1';
				} else {
					$custom_string .= ' AND saleclass = "'.$custom['saleclass'].'"';
				}
			}
			// Build custom make
			if( !empty($custom['make']['value']) ){
				$data = explode(',',$custom['make']['value']);
				$symbol = $this->get_custom_import_symbol($custom['make']['symbol']);
				// Clean white space
				foreach( $data as $value ){
					$clean_data[] = trim($value);
				}
				$search = 'AND make ' .$symbol. ' ("'.implode('","',$clean_data).'") ';
				$custom_string .= $search;
			}
			// Build custom year
			if( !empty($custom['year']['value']) ){
				$data = explode(',',$custom['year']['value']);
				$symbol = $this->get_custom_import_symbol($custom['year']['symbol']);
				// Clean white space
				foreach( $data as $value ){
					$clean_data[] = trim($value);
				}
				if( in_array($custom['year']['symbol'], array(2,3)) ){
					$search = ' AND year ' .$symbol. '"'.$clean_data[0].'" ';
				} else {
					$search = ' AND year ' .$symbol. ' ("'.implode('","',$clean_data).'") ';
				}
				$custom_string .= $search;
			}
		}
		return $custom_string;
	}

	function run_custom_imports(){
		 $skip_custom_import = false;
		// Get list of Dealers
		$query = 'SELECT dealer_id FROM cdp_inventory_import_'.$this->blog.' WHERE blog_id = '.$this->blog.' group by dealer_id';
		$import_id = $this->wpdb->get_results($query, ARRAY_A);
		if( count($import_id) == 1 and !empty($import_id[0]['dealer_id']) ){

			// Delete Sold Vehicles | Custom Not Needed
			$query = 'SELECT ID FROM cdp_inventory_'.$this->blog.' WHERE cdp_inventory_'.$this->blog.'.company_id = '.$this->company_id.' AND cdp_inventory_'.$this->blog.'.blog_id = '.$this->blog.' AND cdp_inventory_'.$this->blog.'.dealer_id = '.$this->dealer['dealer_id'].' AND cdp_inventory_'.$this->blog.'.vms_id NOT IN( SELECT a.vms_id FROM cdp_inventory_import_'.$this->blog.' as a WHERE cdp_inventory_'.$this->blog.'.company_id = a.company_id AND a.blog_id = '.$this->blog.' AND cdp_inventory_'.$this->blog.'.dealer_id = a.dealer_id GROUP BY a.vms_id)';
			$remove = $this->wpdb->get_results($query, ARRAY_A);
			if( $remove ){
				foreach($remove as $row){
					$table = 'cdp_inventory_'.$this->blog;
					$where = array('id'=>$row['ID']);
					$where_format = array('%d');
					$this->wpdb->delete($table,$where,$where_format);
				}
			}
			// Removes Out Dated Vehicles, meaning they need to be updated with new data
			$query = 'SELECT ID FROM cdp_inventory_'.$this->blog.' WHERE cdp_inventory_'.$this->blog.'.company_id = '.$this->company_id.' AND cdp_inventory_'.$this->blog.'.blog_id = '.$this->blog.' AND cdp_inventory_'.$this->blog.'.dealer_id = '.$this->dealer['dealer_id'].' AND cdp_inventory_'.$this->blog.'.vms_id IN( SELECT a.vms_id FROM (SELECT vms_id, updated FROM cdp_inventory_'.$this->blog.' WHERE company_id = '.$this->company_id.' AND blog_id = '.$this->blog.' AND dealer_id = '.$this->dealer['dealer_id'].') as a JOIN cdp_inventory_import_'.$this->blog.' b ON a.vms_id = b.vms_id WHERE a.updated <> b.updated GROUP BY a.vms_id)';
			$remove = $this->wpdb->get_results($query, ARRAY_A);
			if( $remove ){
				foreach($remove as $row){
					$table = 'cdp_inventory_'.$this->blog;
					$where = array('id'=>$row['ID']);
					$where_format = array('%d');
					$this->wpdb->delete($table,$where,$where_format);
				}
			}

			// Get custom imports
			$query = 'SELECT data_value FROM cdp_data WHERE blog_id = '.$this->blog.' AND data_key = "import" AND dealer_id = '.$import_id[0]['dealer_id'];
			$imports = $this->wpdb->get_results($query, ARRAY_A);
			if( empty($imports) ){
				$imports[0] = array('data_value'=>'');
				$skip_custom_import = true;
			}
			foreach($imports as $import){
				$insert_columns = 'blog_id, vms_id, company_id, dealer_id, dealer, stock, vin, saleclass, certified, make, model, model_code, trim, year, askingprice, saleprice, retailprice, msrp, sortprice, pricesort, sale_expire, headline, description, exterior, interior, color_map, body, vehicleclass, tags, drivetrain, doors, engine, transmission, fuel_type, highway, city, odometer, carfax, carfax_image_url, autocheck, video, acode, thumbnail, geo_state, geo_city, geo_zip, location, phone, created, updated, sort, extra';

				$custom_query = $skip_custom_import? '': $this->build_custom_import_query($import['data_value']);
				$query = 'SELECT '.$insert_columns.' FROM cdp_inventory_import_'.$this->blog.' as z WHERE z.vms_id NOT IN( SELECT a.vms_id FROM cdp_inventory_'.$this->blog.' as a WHERE z.company_id = a.company_id and z.dealer_id = a.dealer_id AND a.blog_id = '.$this->blog .' '. $custom_query.' GROUP BY a.vms_id ) '.$custom_query;
				$new_rows = $this->wpdb->get_results($query, ARRAY_A);
				if($new_rows){
					$count = count($new_rows);
					$i = 0;
					do{
						$this->wpdb->insert('cdp_inventory_'.$this->blog.'',$new_rows[$i]);
						$i++;
					}while( $i < $count );
					unset($new_rows);
				}
			}
		}
	}

	function update_live_table(){

		$this->run_custom_imports();
		// Removes Possible Duplicates
		$sql = 'DELETE FROM cdp_inventory_'.$this->blog.' WHERE id in ( SELECT max_id from ( SELECT max(id) as max_id FROM cdp_inventory_'.$this->blog.' WHERE vms_id IN ( SELECT vms_id FROM (SELECT vms_id, count(*) AS duplicates FROM cdp_inventory_'.$this->blog.' WHERE company_id = '.$this->company_id.' AND blog_id = '.$this->blog.' AND dealer_id = '.$this->dealer['dealer_id'].' GROUP BY vms_id ) AS z where z.duplicates > 1 ) GROUP BY vms_id ) AS a )';
		$this->wpdb->query($sql);
		// Update Color
		if( isset($this->settings['vehicle_management_system']['color_mapping']['enable']) && $this->settings['vehicle_management_system']['color_mapping']['enable'] ){
			$this->init_inventory_color_update();
		}
		// Update PriceSort
		$sql = 'UPDATE cdp_inventory_'.$this->blog.' SET pricesort = 99 WHERE sortprice = 0 AND company_id = '.$this->company_id.' AND blog_id = '.$this->blog.' AND dealer_id = '.$this->dealer['dealer_id'];
		$this->wpdb->query($sql);
		// Reset PriceSort
		$sql = 'UPDATE cdp_inventory_'.$this->blog.' SET pricesort = 0 WHERE pricesort = 99 and sortprice <> 0 AND company_id = '.$this->company_id.' AND blog_id = '.$this->blog.' AND dealer_id = '.$this->dealer['dealer_id'];
		$this->wpdb->query($sql);

	}

	function get_external_data($url = ''){
		if( $url ) {
			$this->curl_url = $url;
			$rawData = $this->get_url_string();
			if( !empty($rawData) ){
				$search = array('.', ' ', "\n", "\t", "\r");
				$replace = array('', '', ' ', '', '');
				$data = explode(" ",str_replace($search, $replace, $rawData) );
				return $data;
			}
			return array();
		}
		return array();
	}

	function get_url(){
		// Initiate curl
		$ch = curl_init();
		// Disable SSL verification
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		// Will return the response, if false it print the response
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		// Set the url
		curl_setopt($ch, CURLOPT_URL,$this->curl_url);
		// Execute
		$result=curl_exec($ch);
		// Closing
		curl_close($ch);

		return json_decode($result, true);
	}

	function get_url_string(){
		// Initiate curl
		$ch = curl_init();
		// Disable SSL verification
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		// Will return the response, if false it print the response
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		// Set the url
		curl_setopt($ch, CURLOPT_URL,$this->curl_url);
		// Execute
		$result=curl_exec($ch);
		// Closing
		curl_close($ch);

		return $result;
	}

	function multi_thread_request($nodes){

		$mh = curl_multi_init();
		$curl_array = array();
		foreach($nodes as $i => $url){
			$curl_array[$i] = curl_init($url);
			curl_setopt($curl_array[$i], CURLOPT_RETURNTRANSFER, true);
			curl_multi_add_handle($mh, $curl_array[$i]);
		}
		$running = NULL;
		do{
			usleep(10000);
			curl_multi_exec($mh,$running);
		}while($running > 0);

		$res = array();
		foreach($nodes as $i => $url){
			$res[$i] = json_decode(curl_multi_getcontent($curl_array[$i]), true);
			curl_multi_remove_handle($mh, $curl_array[$i]);
		}

		curl_multi_close($mh);
		return $res;
	}

	function update_database_version(){
		if( $this->settings ){
			update_option( 'cardealerpress_settings' , $this->settings );
		}
	}

	function remove_inventory_tables(){
		$this->wpdb->query('DROP TABLE cdp_inventory_import_'.$this->blog.'');
		$this->wpdb->query('DROP TABLE cdp_inventory_'.$this->blog.'');
		$this->wpdb->query('DROP TABLE cdp_data');
		$this->wpdb->query('DROP TABLE cdp_companies');
	}

}
?>
